<?php
/**
 * Created by Klickfabrik
 * Date: 18.02.15
 * Time: 13:13
 *
 * Developer: Marc Finnern
 * Version: 1.0
 */

class fastbill{
    private $email = '';
    private $apiKey = '';
    private $apiUrl = '';
    private $debug = false;
    private $convert_to_utf8 = false;


    /*
     * Basic init
     */
    public function __construct($_email, $_apiKey, $_apiUrl = FASTBILL_PLUS){
        if($_email != '' && $_apiKey != ''){
            $this->email = $_email;
            $this->apiKey = $_apiKey;
            $this->apiUrl = $_apiUrl;
        } else {
            return false;
        }
    }

    /*
     * option: error-log
     * display error
     */
    public function setDebug($_bool = false){
        if($_bool != ''){
            $this->debug = $_bool;
        } else {
            if($this->debug == true) { return array("RESPONSE" => array("ERROR" => array("Übergabeparameter 1 ist leer!"))); } else  { return false; }
        }
    }


    /*
     * option: UTF8
     * Convert all to UTF-8 data
     */
    public function setConvertToUTF8($_convert_to_utf8 = false){
        if($_convert_to_utf8 != ''){
            $this->convert_to_utf8 = $_convert_to_utf8;
        } else {
            if($this->debug == true) { return array("RESPONSE" => array("ERROR" => array("Übergabeparameter 1 ist leer!"))); } else  { return false; }
        }
    }

    /*
     * reform data to utf-8 data
     */
    private function convertToUTF8($_array){
        foreach($_array AS $key => $val){
            if(is_array($val)){
                $val = $this->convertToUTF8($val);
            } else {
                $val = utf8_encode($val);
            }
            $_array[$key] = $val;
        }

        return $_array;
    }

    /*
     * curl command
     * return result or error
     */
    public function request($_data, $_file = NULL) {
        if($_data) {
            if($this->email != '' && $this->apiKey != '' && $this->apiUrl != '') {
                if($this->convert_to_utf8) { $_data = $this->convertToUTF8($_data); }

                $ch = curl_init();

                $data_string = json_encode($_data);

                if($_file != NULL) { $bodyStr = array("document" => "@".$_file, "httpbody" => $data_string); } else  { $bodyStr = array("httpbody" => $data_string); }

                curl_setopt($ch, CURLOPT_URL, $this->apiUrl);
                curl_setopt($ch, CURLOPT_HTTPHEADER, array('header' => 'Authorization: Basic ' . base64_encode($this->email.':'.$this->apiKey)));
                curl_setopt($ch, CURLOPT_POST, 1);
                curl_setopt($ch, CURLOPT_POSTFIELDS, $bodyStr);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                curl_setopt($ch, CURLOPT_ENCODING, 'gzip');
                curl_setopt($ch, CURLOPT_VERBOSE, 1);

                $exec = curl_exec($ch);

                $result = json_decode($exec,true);

                curl_close($ch);

                return $result;
            } else {
                if($this->debug == true) { return array("RESPONSE" => array("ERROR" => array("Email und/oder APIKey und/oder APIURL Fehlen!"))); } else  { return false; }
            }
        } else {
            if($this->debug == true) { return array("RESPONSE" => array("ERROR" => array("Übergabeparameter 1 ist leer!"))); } else  { return false; }
        }
    }




    /*
     * Request: check API
     * @return: true / false
     */
    public function checkAPICredentials(){
        $ret = $this->request(array('SERVICE' => 'customer.get'));

        if(isset($ret['RESPONSE']['ERRORS'])) { return false; } else { return true; }
    }


    /*
     * Request: get client list
     * @return: true / false
     */
    public function getClients(){
        $ret = $this->request(array('SERVICE' => 'customer.get'));

        if(isset($ret['RESPONSE']['ERRORS'])) { return false; } else { return $ret; }
    }


    /*
     * Request: get templates
     * @return: true/false
     */
    public function getTemplates(){
        $request = array(
            "SERVICE"   => "template.get",
        );

        $ret = $this->request($request);

        if(isset($ret['RESPONSE']['ERRORS'])) {
            return false;
        } else {
            if(isset($ret['RESPONSE']['TEMPLATES']) && !empty($ret['RESPONSE']['TEMPLATES'])){
                $return = array(
                    "status"    => true,
                    "RESPONSE"  => $ret['RESPONSE']['TEMPLATES']
                );
            } else {
                $return = array(
                    "status"    => false
                );
            }

            return $return;
        }
    }


    /*
     * Request: check if client exist
     * @return: true/false
     */
    public function checkClientExist($search){
        $request = array(
            "SERVICE"   => "customer.get",
            "FILTER"    => array(
                "TERM"  => $search
            )
        );

        $ret = $this->request($request);

        if(isset($ret['RESPONSE']['ERRORS'])) {
            return false;
        } else {
            if(isset($ret['RESPONSE']['CUSTOMERS']) && !empty($ret['RESPONSE']['CUSTOMERS'])){
                $return = array(
                    "status"    => true,
                    "RESPONSE"  => $ret['RESPONSE']['CUSTOMERS'][0]
                );
            } else {
                $return = array(
                    "status"    => false
                );
            }

            return $return;
        }
    }


    /*
     * Request: check if Invoice exist
     * @return: true/false
     */
    public function checkInvoice($search){

        // create filter
        if(!is_array($search)){
            return false;
        } else {
            $filter = array();
            foreach($search as $tag => $value){
                $filter[strtoupper($tag)] = $value;
            }
        }

        $invoice = array(
            "SERVICE" => "invoice.get",
            "FILTER" => $filter,
        );

        $ret = $this->request($invoice);


        if(isset($ret['RESPONSE']['ERRORS'])) {
            return false;
        } else {
            if(isset($ret['RESPONSE']['INVOICES']) && !empty($ret['RESPONSE']['INVOICES'])){
                $return = array(
                    "status"    => true,
                    "RESPONSE"  => $ret['RESPONSE']['INVOICES'][0]
                );
            } else {
                $return = array(
                    "status"    => false
                );
            }

            return $return;
        }

    }





    /*
     * helper: showArray($arr=array)
     * show array display an array like print_r but nice
     */
    public function showArray($arr=array())  {
        echo '<pre>'.print_r($arr,true).'</pre>';
    }

    /*
     * helper: get ISO
     *
     */
    public function getCountry($country){

        $searchLength = strlen($country);

        $countries = array (
            'AF' => 'Afghanistan',
            'AX' => 'Aland Islands',
            'AL' => 'Albania',
            'DZ' => 'Algeria',
            'AS' => 'American Samoa',
            'AD' => 'Andorra',
            'AO' => 'Angola',
            'AI' => 'Anguilla',
            'AQ' => 'Antarctica',
            'AG' => 'Antigua And Barbuda',
            'AR' => 'Argentina',
            'AM' => 'Armenia',
            'AW' => 'Aruba',
            'AU' => 'Australia',
            'AT' => 'Austria',
            'AZ' => 'Azerbaijan',
            'BS' => 'Bahamas',
            'BH' => 'Bahrain',
            'BD' => 'Bangladesh',
            'BB' => 'Barbados',
            'BY' => 'Belarus',
            'BE' => 'Belgium',
            'BZ' => 'Belize',
            'BJ' => 'Benin',
            'BM' => 'Bermuda',
            'BT' => 'Bhutan',
            'BO' => 'Bolivia',
            'BA' => 'Bosnia And Herzegovina',
            'BW' => 'Botswana',
            'BV' => 'Bouvet Island',
            'BR' => 'Brazil',
            'IO' => 'British Indian Ocean Territory',
            'BN' => 'Brunei Darussalam',
            'BG' => 'Bulgaria',
            'BF' => 'Burkina Faso',
            'BI' => 'Burundi',
            'KH' => 'Cambodia',
            'CM' => 'Cameroon',
            'CA' => 'Canada',
            'CV' => 'Cape Verde',
            'KY' => 'Cayman Islands',
            'CF' => 'Central African Republic',
            'TD' => 'Chad',
            'CL' => 'Chile',
            'CN' => 'China',
            'CX' => 'Christmas Island',
            'CC' => 'Cocos (Keeling) Islands',
            'CO' => 'Colombia',
            'KM' => 'Comoros',
            'CG' => 'Congo',
            'CD' => 'Congo, Democratic Republic',
            'CK' => 'Cook Islands',
            'CR' => 'Costa Rica',
            'CI' => 'Cote D\'Ivoire',
            'HR' => 'Croatia',
            'CU' => 'Cuba',
            'CY' => 'Cyprus',
            'CZ' => 'Czech Republic',
            'DK' => 'Denmark',
            'DJ' => 'Djibouti',
            'DM' => 'Dominica',
            'DO' => 'Dominican Republic',
            'EC' => 'Ecuador',
            'EG' => 'Egypt',
            'SV' => 'El Salvador',
            'GQ' => 'Equatorial Guinea',
            'ER' => 'Eritrea',
            'EE' => 'Estonia',
            'ET' => 'Ethiopia',
            'FK' => 'Falkland Islands (Malvinas)',
            'FO' => 'Faroe Islands',
            'FJ' => 'Fiji',
            'FI' => 'Finland',
            'FR' => 'France',
            'GF' => 'French Guiana',
            'PF' => 'French Polynesia',
            'TF' => 'French Southern Territories',
            'GA' => 'Gabon',
            'GM' => 'Gambia',
            'GE' => 'Georgia',
            'DE' => 'Germany',
            'GH' => 'Ghana',
            'GI' => 'Gibraltar',
            'GR' => 'Greece',
            'GL' => 'Greenland',
            'GD' => 'Grenada',
            'GP' => 'Guadeloupe',
            'GU' => 'Guam',
            'GT' => 'Guatemala',
            'GG' => 'Guernsey',
            'GN' => 'Guinea',
            'GW' => 'Guinea-Bissau',
            'GY' => 'Guyana',
            'HT' => 'Haiti',
            'HM' => 'Heard Island & Mcdonald Islands',
            'VA' => 'Holy See (Vatican City State)',
            'HN' => 'Honduras',
            'HK' => 'Hong Kong',
            'HU' => 'Hungary',
            'IS' => 'Iceland',
            'IN' => 'India',
            'ID' => 'Indonesia',
            'IR' => 'Iran, Islamic Republic Of',
            'IQ' => 'Iraq',
            'IE' => 'Ireland',
            'IM' => 'Isle Of Man',
            'IL' => 'Israel',
            'IT' => 'Italy',
            'JM' => 'Jamaica',
            'JP' => 'Japan',
            'JE' => 'Jersey',
            'JO' => 'Jordan',
            'KZ' => 'Kazakhstan',
            'KE' => 'Kenya',
            'KI' => 'Kiribati',
            'KR' => 'Korea',
            'KW' => 'Kuwait',
            'KG' => 'Kyrgyzstan',
            'LA' => 'Lao People\'s Democratic Republic',
            'LV' => 'Latvia',
            'LB' => 'Lebanon',
            'LS' => 'Lesotho',
            'LR' => 'Liberia',
            'LY' => 'Libyan Arab Jamahiriya',
            'LI' => 'Liechtenstein',
            'LT' => 'Lithuania',
            'LU' => 'Luxembourg',
            'MO' => 'Macao',
            'MK' => 'Macedonia',
            'MG' => 'Madagascar',
            'MW' => 'Malawi',
            'MY' => 'Malaysia',
            'MV' => 'Maldives',
            'ML' => 'Mali',
            'MT' => 'Malta',
            'MH' => 'Marshall Islands',
            'MQ' => 'Martinique',
            'MR' => 'Mauritania',
            'MU' => 'Mauritius',
            'YT' => 'Mayotte',
            'MX' => 'Mexico',
            'FM' => 'Micronesia, Federated States Of',
            'MD' => 'Moldova',
            'MC' => 'Monaco',
            'MN' => 'Mongolia',
            'ME' => 'Montenegro',
            'MS' => 'Montserrat',
            'MA' => 'Morocco',
            'MZ' => 'Mozambique',
            'MM' => 'Myanmar',
            'NA' => 'Namibia',
            'NR' => 'Nauru',
            'NP' => 'Nepal',
            'NL' => 'Netherlands',
            'AN' => 'Netherlands Antilles',
            'NC' => 'New Caledonia',
            'NZ' => 'New Zealand',
            'NI' => 'Nicaragua',
            'NE' => 'Niger',
            'NG' => 'Nigeria',
            'NU' => 'Niue',
            'NF' => 'Norfolk Island',
            'MP' => 'Northern Mariana Islands',
            'NO' => 'Norway',
            'OM' => 'Oman',
            'PK' => 'Pakistan',
            'PW' => 'Palau',
            'PS' => 'Palestinian Territory, Occupied',
            'PA' => 'Panama',
            'PG' => 'Papua New Guinea',
            'PY' => 'Paraguay',
            'PE' => 'Peru',
            'PH' => 'Philippines',
            'PN' => 'Pitcairn',
            'PL' => 'Poland',
            'PT' => 'Portugal',
            'PR' => 'Puerto Rico',
            'QA' => 'Qatar',
            'RE' => 'Reunion',
            'RO' => 'Romania',
            'RU' => 'Russian Federation',
            'RW' => 'Rwanda',
            'BL' => 'Saint Barthelemy',
            'SH' => 'Saint Helena',
            'KN' => 'Saint Kitts And Nevis',
            'LC' => 'Saint Lucia',
            'MF' => 'Saint Martin',
            'PM' => 'Saint Pierre And Miquelon',
            'VC' => 'Saint Vincent And Grenadines',
            'WS' => 'Samoa',
            'SM' => 'San Marino',
            'ST' => 'Sao Tome And Principe',
            'SA' => 'Saudi Arabia',
            'SN' => 'Senegal',
            'RS' => 'Serbia',
            'SC' => 'Seychelles',
            'SL' => 'Sierra Leone',
            'SG' => 'Singapore',
            'SK' => 'Slovakia',
            'SI' => 'Slovenia',
            'SB' => 'Solomon Islands',
            'SO' => 'Somalia',
            'ZA' => 'South Africa',
            'GS' => 'South Georgia And Sandwich Isl.',
            'ES' => 'Spain',
            'LK' => 'Sri Lanka',
            'SD' => 'Sudan',
            'SR' => 'Suriname',
            'SJ' => 'Svalbard And Jan Mayen',
            'SZ' => 'Swaziland',
            'SE' => 'Sweden',
            'CH' => 'Switzerland',
            'SY' => 'Syrian Arab Republic',
            'TW' => 'Taiwan',
            'TJ' => 'Tajikistan',
            'TZ' => 'Tanzania',
            'TH' => 'Thailand',
            'TL' => 'Timor-Leste',
            'TG' => 'Togo',
            'TK' => 'Tokelau',
            'TO' => 'Tonga',
            'TT' => 'Trinidad And Tobago',
            'TN' => 'Tunisia',
            'TR' => 'Turkey',
            'TM' => 'Turkmenistan',
            'TC' => 'Turks And Caicos Islands',
            'TV' => 'Tuvalu',
            'UG' => 'Uganda',
            'UA' => 'Ukraine',
            'AE' => 'United Arab Emirates',
            'GB' => 'United Kingdom',
            'US' => 'United States',
            'UM' => 'United States Outlying Islands',
            'UY' => 'Uruguay',
            'UZ' => 'Uzbekistan',
            'VU' => 'Vanuatu',
            'VE' => 'Venezuela',
            'VN' => 'Viet Nam',
            'VG' => 'Virgin Islands, British',
            'VI' => 'Virgin Islands, U.S.',
            'WF' => 'Wallis And Futuna',
            'EH' => 'Western Sahara',
            'YE' => 'Yemen',
            'ZM' => 'Zambia',
            'ZW' => 'Zimbabwe',
        );

        foreach($countries as $ISO => $name){
            if($searchLength == 2){
                if(strtolower($ISO) == strtolower($country)){
                    return $ISO;
                }
            } else {
                if(strtolower($name) == strtolower($country)){
                    return $ISO;
                }
            }
        }
    }
}