<?php
/**
 * Created by Klickfabrik
 * Date: 23.02.15
 * Time: 08:35
 *
 * Developer: Marc Finnern
 * Version: 1.0
 */


/*
 * check if company exist and company-name has
 * a minimum of length.
 * return: string
 */
function companycheck($data){
    if(isset($data['ORGANIZATION']) && !empty($data['ORGANIZATION'])){
        $minLength = get_option(kf_fbapi_field.'export_company_minlength') !== false ? get_option(kf_fbapi_field.'export_company_minlength') : 5;
        if(strlen($data['ORGANIZATION']) <= $minLength){
            $tableContent = __('Company',kf_fbapi_plugin).": <mark>".$data['ORGANIZATION']."</mark><br/>";
        } else {
            $tableContent = __('Company',kf_fbapi_plugin).": ".$data['ORGANIZATION']."<br/>";
        }

        return $tableContent;
    } else {
        return false;
    }
}

/*
 * load all order-information from wordpress
 * return: array()
 */
function getBillingData($order_id) {
    $address = array(
        "FIRST_NAME"        => ucfirst(get_post_meta( $order_id, '_billing_first_name', true )),
        "LAST_NAME"         => ucfirst(get_post_meta( $order_id, '_billing_last_name', true )),
        //"SALUTATION"        => get_post_meta( $order_id, '', true ), // TODO create a function for wooCommerce
        "EMAIL"             => get_post_meta( $order_id, '_billing_email', true ),
        "PHONE"             => get_post_meta( $order_id, '_billing_phone', true ),

        "ORGANIZATION"      => get_post_meta( $order_id, '_billing_company', true ),
        "ADDRESS"           => get_post_meta( $order_id, '_billing_address_1', true ),
        "ADDRESS_2"         => get_post_meta( $order_id, '_billing_address_2', true ),
        "CITY"              => get_post_meta( $order_id, '_billing_city', true ),
        "STATE"             => get_post_meta( $order_id, '_billing_state', true ),
        "ZIPCODE"           => get_post_meta( $order_id, '_billing_postcode', true ),
        "COUNTRY_CODE"      => get_post_meta( $order_id, '_billing_country', true ),
    );

    return $address;
}

/*
 * export data to fastbill
 */
function apiExport($id,$debug=false){
    // Order-ID
    $order_id   = $id;

    // init
    $fastBill   = new fastbill(get_option(kf_fbapi_field.'api_email'),get_option(kf_fbapi_field.'api_key'),FASTBILL_PLUS);

    // search client
    $userData   = getBillingData($order_id);

    // check client
    $userData['CUSTOMER_TYPE'] = !empty($userData['ORGANIZATION']) ? "business" : "consumer";

    /* *********************************************************************
     * Client check: create client if not exist
     * *********************************************************************/

    switch($userData['CUSTOMER_TYPE']){
        case "business":
            $apiCheckClientResult = $fastBill->checkClientExist($userData['ORGANIZATION']);
            break;
        case "consumer":
            $apiCheckClientResult = $fastBill->checkClientExist($userData['LAST_NAME']);
            break;
    }

    // not exist
    if($apiCheckClientResult['status'] == false){
        $clientData = array(
            "SERVICE"       => "customer.create",
            "DATA"          => $userData
        );
        $response = $fastBill->request($clientData);

        if($response['RESPONSE']['STATUS'] == "success"){
            $messages[] = "Client created - Type: ".$userData['CUSTOMER_TYPE'];
            $apiCheckClientResult['RESPONSE']['CUSTOMER_ID'] = $response['RESPONSE']['CUSTOMER_ID'];
        }

        if($debug)
            $fastBill->showArray($response);
    }


    /* *********************************************************************
     * Invoice check: create client if not exist
     * *********************************************************************/

    // options infos
    $invoiceTitle       = str_replace("%NUMBER%",$order_id,get_option(kf_fbapi_field.'api_invoice_title'));
    $invoiceTemplate    = get_option(kf_fbapi_field.'api_invoice_template');
    $invoiceCurrency    = get_option(kf_fbapi_field.'api_invoice_currency_code');
    $invoiceExist       = false;
    $invoicePaid        = false;


    $apiGetInvoiceResult = $fastBill->checkInvoice(
        array(
            "invoice_title" => $invoiceTitle,
            "CUSTOMER_ID"   => $apiCheckClientResult['RESPONSE']['CUSTOMER_ID']
        )
    );


    // not exist
    if($apiGetInvoiceResult['status'] == false){

        // WooCommerce API
        $order  = new WC_Order( $order_id );
        $tax    = new WC_Tax();

        // get tax
        $taxData = array_shift($tax->find_rates(array("country"=>$userData["COUNTRY_CODE"])));
        $tax_percent = isset($taxData['rate']) ? $taxData['rate'] : 0;


        // get WooCommerce Items
        $items  = $order->get_items();


        // build Items for FastBill
        $invoiceDataItems = array();
        foreach($items as $item){
            if(!empty($item['product_id'])){
                $unit_price     = str_replace(",",".",array_shift($item['item_meta']['_line_subtotal']));
                $vat_percent    = $unit_price != 0 ? $unit_price : 0;

                $invoiceDataItems[] = array(
                    "ARTICLE_NUMBER"=> $item['product_id'],
                    "DESCRIPTION"   => $item['name'],
                    "QUANTITY"      => array_shift($item['item_meta']['_qty']),
                    "UNIT_PRICE"    => strpos(".",$unit_price)!==false ? $unit_price : $unit_price.".00",
                    "VAT_PERCENT"   => $vat_percent,
                );
            }
        }


        // fastBill API
        $invoiceDate    = date("Y-m-d",strtotime($order->order_date));
        $invoiceData = array(
            "SERVICE"   => "invoice.create",
            "DATA"      => array(
                "CUSTOMER_ID"   => $apiCheckClientResult['RESPONSE']['CUSTOMER_ID'],
                "INVOICE_TITLE" => $invoiceTitle,
                "INVOICE_DATE"  => $invoiceDate,
                //"DELIVERY_DATE" => date("Y-m-d",strtotime($order->order_date)),
                "CURRENCY_CODE" => $invoiceCurrency,        // Standardwährung
                "TEMPLATE_ID"   => $invoiceTemplate,        // From template.get
                "ITEMS"         => $invoiceDataItems,       // billing items
            ),
        );

        // create invoice
        $response = $fastBill->request($invoiceData);

        if($response['RESPONSE']['STATUS'] == "success"){
            // create final invoice
            $invoiceId      = $response['RESPONSE']['INVOICE_ID'];

            // Message
            $messages[] = "Invoice draft created - ID: ".$invoiceId;

        }

        if($debug)
            $fastBill->showArray($response);
    } else {
        // if exist
        $invoiceId      = $apiGetInvoiceResult['RESPONSE']['INVOICE_ID'];
        $invoiceDate    = $apiGetInvoiceResult['RESPONSE']['INVOICE_DATE'];
        $invoiceExist   = isset($apiGetInvoiceResult['RESPONSE']['DOCUMENT_URL']) ? true : false;
        $invoicePaid    = $apiGetInvoiceResult['RESPONSE']['PAID_DATE'] != "0000-00-00 00:00:00" ? true : false;

        if($debug)
            $fastBill->showArray($apiGetInvoiceResult);
    }

    /* *********************************************************************
     * Invoice final: create a final invoice
     * *********************************************************************/
    $invoiceType        = get_option(kf_fbapi_field.'api_invoice_type');


    if($invoiceType == "invoice_final" && $invoiceExist == false){
        $invoiceUpdateData = array(
            "SERVICE"   => "invoice.complete",
            "DATA"      => array(
                "INVOICE_ID" => $invoiceId
            ),
        );
        $response = $fastBill->request($invoiceUpdateData);


        if($response['RESPONSE']['STATUS'] == "success"){
            // Message
            $messages[] = "Invoice final created - ID: ".$invoiceId;
        }

        if($debug)
            $fastBill->showArray($response);
    }


    /* *********************************************************************
     * Invoice: set paid
     * *********************************************************************/
    $wooCommerceStatus = esc_attr( get_option(kf_fbapi_field.'api_woocommerce_status'));
    if($wooCommerceStatus == "wc-completed" && $invoicePaid == false){
        $invoiceUpdateData = array(
            "SERVICE"   => "invoice.setpaid",
            "DATA"      => array(
                "INVOICE_ID"    => $invoiceId,
                "PAID_DATE"     => $invoiceDate
            ),
        );
        $response = $fastBill->request($invoiceUpdateData);
        if($response['RESPONSE']['STATUS'] == "success"){
            // Message
            $messages[] = "Invoice paid - ID: ".$invoiceId;
        }

        if($debug)
            $fastBill->showArray($response);

    }


    /* *********************************************************************
     * Invoice: save status database
     * *********************************************************************/
    $key = "kf_fbapi_invoice_".$order_id;
    if ( get_option( $key ) !== false ) {
        update_option( $key, $invoiceType );
    } else {
        add_option( $key, $invoiceType, null, 'no' );
    }


    /* *********************************************************************
     * Invoice: get final Document-URL
     * *********************************************************************/
    if($invoiceExist == false){
        $apiGetInvoiceResult = $fastBill->checkInvoice(
            array(
                "INVOICE_ID"    => $invoiceId
            )
        );
        $invoiceDocument = $apiGetInvoiceResult['RESPONSE']['DOCUMENT_URL'];
    } else {
        $invoiceDocument = $apiGetInvoiceResult['RESPONSE']['DOCUMENT_URL'];
    }


    /* *********************************************************************
     * Invoice: save document database
     * *********************************************************************/

    if(isset($invoiceDocument) && !empty($invoiceDocument)){
        $key = "kf_fbapi_invoice_".$order_id."_document";
        if ( get_option( $key ) !== false ) {
            update_option( $key, $invoiceDocument );
        } else {
            add_option( $key, $invoiceDocument, null, 'no' );
        }
    }


    return $messages;
}